/**
 * @type {{play: (msg:string) => void, error: (err:string) => void, success: (msg:string) => void, progress: (msg:string) => void, cancel:() => void}}
 */

// @ts-ignore
const spinner = new LoadingAnimation();

// @ts-ignore
const IS_IN_DEVELOPMENT = OpenIndesign.isDev,
  mapResMsgToClass = {
    success: "success-text",
    error: "error-text",
  },
  LOG_PANE_ID = "#logAreaOpenIndd",
  LOG_TAB_HEADERS = "#log-tabs-headers",
  MAIN_LOG_PANEL = "#logArea";

$(function () {
  // Used for selecting tabs
  // @ts-ignore
  $("#log-tabs").tabs();

  if (IS_IN_DEVELOPMENT) {
    console.log("IS IN DEV: ", IS_IN_DEVELOPMENT);
    $(LOG_TAB_HEADERS).show();
    $(LOG_PANE_ID).show();
  }
});

/**
 * Writes to log panel. If in dev writes to the log panel specified else writes to the main log panel
 * @param {string} msg
 * @param {string} panel
 * @param {string | undefined} styles
 * @param {HTMLElement | undefined} customElement
 */
const writePanel = (msg, panel, styles, customElement) => {
  const _panel = IS_IN_DEVELOPMENT ? panel : MAIN_LOG_PANEL;

  if (customElement === undefined) {
    const space = document.createElement("br");
    const line = document.createElement("span");
    line.textContent = msg;

    if (styles !== undefined) {
      line.className = styles;
    }

    $(_panel).append(line);
    $(_panel).append(space);
  } else {
    $(_panel).append(customElement);
  }
};

/**
 *
 * @param {string} msg
 * @param {boolean | undefined} clear
 * @param {string | undefined} color
 */
const handleLogEvent = (msg, clear, color) => {
  if (clear == true) {
    $(LOG_PANE_ID).empty();
  }

  const style = color !== undefined ? mapResMsgToClass[color] : undefined;

  writePanel(msg, LOG_PANE_ID, style, undefined);
};

/**
 *
 * @param {string} err
 * @param {boolean | undefined} clear
 */
const handleErrorEvent = (err, clear) => {
  if (clear == true) {
    $(LOG_PANE_ID).empty();
  }

  writePanel(err, LOG_PANE_ID, "error", undefined);
};

const markEndAction = (msg) => {
  const div = document.createElement("div");
  div.className = "mark-end-of-action";

  const span = document.createElement("span");
  span.className = "log-line";
  span.textContent = msg;

  div.appendChild(span);

  writePanel("", LOG_PANE_ID, undefined, div);
};

/**
 *
 * @param {'loading' | 'success' | 'warning'  | 'error' | 'progress'} status
 * @param {string | undefined} msg
 */
const handleLoadingEvent = (status, msg) => {
  console.log("status: ", status);
  console.log("msg: ", msg);

  switch (status) {
    case "loading":
      spinner.play(msg);
      break;

    case "success":
      spinner.success(msg);
      break;

    case "error":
      spinner.error(msg);
      break;

    case "progress":
      spinner.progress(msg);
      break;

    default:
      break;
  }
};

//@ts-ignore
OpenIndesign.add_log_event_listener(handleLogEvent);

//@ts-ignore
OpenIndesign.add_error_event_listener(handleErrorEvent);

//@ts-ignore
OpenIndesign.add_loading_event_listener(handleLoadingEvent);

const cross = document.getElementById("close-wrapper");

cross.addEventListener("click", (e) => {
  e.preventDefault();
  spinner.cancel();
});
